=== modified file 'daemons/poppy-sftp.tac'
--- daemons/poppy-sftp.tac	2011-02-23 15:16:38 +0000
+++ daemons/poppy-sftp.tac	2011-05-09 14:31:50 +0000
@@ -5,20 +5,26 @@
 #     twistd -noy sftp.tac
 # or similar.  Refer to the twistd(1) man page for details.
 
+import atexit
 import logging
 
 from twisted.application import service
 from twisted.conch.interfaces import ISession
 from twisted.conch.ssh import filetransfer
 from twisted.cred.portal import IRealm, Portal
+from twisted.internet import task
+from twisted.internet.error import AlreadyCancelled
 from twisted.protocols.policies import TimeoutFactory
 from twisted.python import components
 from twisted.web.xmlrpc import Proxy
 
+from zope.component import getUtility
+from zope.component.interfaces import ComponentLookupError
 from zope.interface import implements
 
 from canonical.config import config
 from canonical.launchpad.daemons import readyservice
+from canonical.launchpad.interfaces.gpghandler import IGPGHandler
 from canonical.launchpad.scripts import execute_zcml_for_scripts
 
 from lp.poppy import get_poppy_root
@@ -69,6 +75,38 @@
     return SFTPServer(avatar, get_poppy_root())
 
 
+class GPGHandlerJob:
+    """Manages the twisted job to touch the files in the gpgconfig directory."""
+    def __init__(self):
+        self._gpghandler_job = None
+        # stop the GPGHandler job on normal termination.
+        atexit.register(self._stopGPGHandlerJob)
+        # start the GPGHandler job
+        self._scheduleGPGHandlerJob()
+
+    def _scheduleGPGHandlerJob(self, touch_interval=12 * 3600):
+        # Create a job to touch the GPGHandler home directory every so often
+        # so that it does not get cleaned up by any reaper scripts which look
+        # at time last modified.
+
+        self._stopGPGHandlerJob()
+        try:
+            self._gpghandler_job = task.LoopingCall(
+                getUtility(IGPGHandler).touchConfigurationDirectory)
+            return self._gpghandler_job.start(touch_interval)
+        except ComponentLookupError:
+            # No GPGHandler so no need to start the job.
+            pass
+
+    def _stopGPGHandlerJob(self):
+        try:
+            if self._gpghandler_job and self._gpghandler_job.running:
+                self._gpghandler_job.stop()
+        except AlreadyCancelled:
+            # So we're already cancelled, meh.
+            pass
+
+
 # Connect Python logging to Twisted's logging.
 from lp.services.twistedsupport.loggingsupport import set_up_tacfile_logging
 set_up_tacfile_logging("poppy-sftp", logging.INFO)
@@ -109,5 +147,8 @@
 # We need Zope for looking up the GPG utilities.
 execute_zcml_for_scripts()
 
+# Set up the GPGHandler job
+gpgHandlerJob = GPGHandlerJob()
+
 # Service that announces when the daemon is ready
 readyservice.ReadyService().setServiceParent(application)

=== modified file 'lib/canonical/launchpad/interfaces/gpghandler.py'
--- lib/canonical/launchpad/interfaces/gpghandler.py	2011-05-09 01:57:28 +0000
+++ lib/canonical/launchpad/interfaces/gpghandler.py	2011-05-09 14:31:50 +0000
@@ -269,6 +269,13 @@
         """
         #FIXME RBC: this should be a zope test cleanup thing per SteveA.
 
+    def touchConfigurationDirectory():
+        """Touch the home directory and all files within.
+
+        This function is called so that the configuration directory does not
+        get cleaned up by any reaper scripts which look at time last modified.
+        """
+
 
 class IPymeSignature(Interface):
     """pyME signature container."""

=== modified file 'lib/canonical/launchpad/utilities/ftests/test_gpghandler.py'
--- lib/canonical/launchpad/utilities/ftests/test_gpghandler.py	2011-05-09 01:57:28 +0000
+++ lib/canonical/launchpad/utilities/ftests/test_gpghandler.py	2011-05-09 14:31:50 +0000
@@ -3,8 +3,16 @@
 
 import unittest
 
+from calendar import timegm
+from datetime import (
+    datetime,
+    timedelta,
+    )
+from pytz import UTC
 import gpgme
+import os
 from zope.component import getUtility
+from zope.security.proxy import removeSecurityProxy
 
 from canonical.launchpad.ftests import (
     ANONYMOUS,
@@ -175,12 +183,27 @@
             self.gpg_handler.importKeyringFile(ring)
         self.assertEqual(self.gpg_handler.checkTrustDb(), 0)
 
-
-def test_suite():
-    return unittest.TestLoader().loadTestsFromName(__name__)
-
-
-if __name__ == "__main__":
-    unittest.main(defaultTest=test_suite())
-
-
+    def testHomeDirectoryJob(self):
+        """Does the job to touch the home work."""
+        gpghandler = getUtility(IGPGHandler)
+        naked_gpghandler = removeSecurityProxy(gpghandler)
+
+        # Get a list of all the files in the home directory.
+        files_to_check = [os.path.join(naked_gpghandler.home, f)
+            for f in os.listdir(naked_gpghandler.home)]
+        files_to_check.append(naked_gpghandler.home)
+        self.assertTrue(len(files_to_check) > 1)
+
+        # Set the last modified times to 12 hours ago
+        nowless12 = (datetime.now(UTC) - timedelta(hours=12)).utctimetuple()
+        lm_time = timegm(nowless12)
+        for fname in files_to_check:
+            os.utime(fname, (lm_time, lm_time))
+
+        # Touch the files and re-check the last modified times.
+        gpghandler.touchConfigurationDirectory()
+        second_last_modified_times = dict(
+            (fname, os.path.getmtime(fname)) for fname in files_to_check)
+        for fname in files_to_check:
+            self.assertTrue(
+                lm_time + 12 * 3600 <= second_last_modified_times[fname])

=== modified file 'lib/canonical/launchpad/utilities/gpghandler.py'
--- lib/canonical/launchpad/utilities/gpghandler.py	2011-05-09 01:57:28 +0000
+++ lib/canonical/launchpad/utilities/gpghandler.py	2011-05-09 14:31:50 +0000
@@ -72,7 +72,7 @@
         """Create a new directory containing the required configuration.
 
         This method is called inside the class constructor and genereates
-        a new directory (name ramdomly generated with the 'gpg-' prefix)
+        a new directory (name randomly generated with the 'gpg-' prefix)
         containing the proper file configuration and options.
 
         Also installs an atexit handler to remove the directory on normal
@@ -118,6 +118,16 @@
             if os.path.exists(filename):
                 os.remove(filename)
 
+    def touchConfigurationDirectory(self):
+        """See IGPGHandler."""
+        os.utime(self.home, None)
+        for file in os.listdir(self.home):
+            try:
+                os.utime(os.path.join(self.home, file), None)
+            except OSError:
+                # The file has been deleted.
+                pass
+
     def verifySignature(self, content, signature=None):
         """See IGPGHandler."""
         try:

